@@ -1,29 +1,32 @@
 /* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2010 Red Hat, Inc.
+ * Copyright (C) 2008-2013 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2013 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
+ * USA
  */
 
 #if !defined (__IBUS_H_INSIDE__) && !defined (IBUS_COMPILATION)
 #error "Only <ibus.h> can be included directly"
 #endif
 
+#ifndef __IBUS_SHARE_H_
+#define __IBUS_SHARE_H_
+
 /**
  * SECTION: ibusshare
  * @short_description: Shared utility functions and definition.
@@ -33,11 +36,14 @@
  * which are shared among ibus component and services.
  */
 
-#ifndef __IBUS_SHARE_H_
-#define __IBUS_SHARE_H_
-
 #include <glib.h>
 
+#ifdef IBUS_DISABLE_DEPRECATION_WARNINGS
+#define IBUS_DEPRECATED
+#else
+#define IBUS_DEPRECATED G_DEPRECATED
+#endif
+
 /**
  * IBUS_SERVICE_IBUS:
  *
@@ -291,9 +297,7 @@ gboolean         ibus_key_event_from_string
 /**
  * ibus_init:
  *
- * Init the ibus types.
- *
- * It is actually a wrapper of g_type_init().
+ * Initialize the ibus types.
  */
 void             ibus_init              (void);
 
