@@ -8,88 +8,105 @@
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
-# version 2 of the License, or (at your option) any later version.
+# version 2.1 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Lesser General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
-# License along with this program; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place, Suite 330,
-# Boston, MA  02111-1307  USA
+# License along with this library; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
+# USA
 
 __all__ = (
     "KeyboardShortcutSelection",
     "KeyboardShortcutSelectionDialog",
 );
 
-import gobject
-import gtk
-from gtk import gdk
-from gtk import keysyms
+from gi.repository import Gdk
+from gi.repository import GObject
+from gi.repository import Gtk
+from gi.repository import IBus
+from gi.repository import Pango
+
 from i18n import _, N_
 
 MAX_HOTKEY = 6
 
-class KeyboardShortcutSelection(gtk.VBox):
+class KeyboardShortcutSelection(Gtk.VBox):
     def __init__(self, shortcuts = None):
         super(KeyboardShortcutSelection, self).__init__()
         self.__init_ui()
         self.set_shortcuts(shortcuts)
 
     def __init_ui(self):
-        # label = gtk.Label(_("Keyboard shortcuts:"))
-        # label.set_justify(gtk.JUSTIFY_LEFT)
+        # label = Gtk.Label(_("Keyboard shortcuts:"))
+        # label.set_justify(Gtk.Justification.LEFT)
         # label.set_alignment(0.0, 0.5)
         # self.pack_start(label, False, True, 4)
 
         # shortcuts view
-        self.__shortcut_view = gtk.TreeView(gtk.ListStore(gobject.TYPE_STRING))
+        self.__shortcut_view = Gtk.TreeView(Gtk.ListStore(GObject.TYPE_STRING))
         self.__shortcut_view.set_size_request(-1, 100)
-        renderer = gtk.CellRendererText()
-        column = gtk.TreeViewColumn(_("Keyboard shortcuts"), renderer, text = 0)
+        renderer = Gtk.CellRendererText()
+        column = Gtk.TreeViewColumn(_("Keyboard shortcuts"), renderer, text = 0)
         self.__shortcut_view.append_column(column)
         self.__shortcut_view.connect("cursor-changed", self.__shortcut_view_cursor_changed_cb)
-        scrolledwindow = gtk.ScrolledWindow()
-        scrolledwindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        scrolledwindow = Gtk.ScrolledWindow()
+        scrolledwindow.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
         scrolledwindow.add(self.__shortcut_view)
-        scrolledwindow.set_shadow_type(gtk.SHADOW_IN)
+        scrolledwindow.set_shadow_type(Gtk.ShadowType.IN)
         self.pack_start(scrolledwindow, True, True, 4)
 
         # key code
-        hbox = gtk.HBox()
-        label = gtk.Label(_("Key code:"))
-        label.set_justify(gtk.JUSTIFY_LEFT)
+        hbox = Gtk.HBox()
+        label = Gtk.Label(_("Key code:"))
+        label.set_justify(Gtk.Justification.LEFT)
         label.set_alignment(0.0, 0.5)
         hbox.pack_start(label, False, True, 4)
 
-        self.__keycode_entry = gtk.Entry()
+        self.__keycode_entry = Gtk.Entry()
         self.__keycode_entry.connect("notify::text", self.__keycode_entry_notify_cb)
         hbox.pack_start(self.__keycode_entry, True, True, 4)
-        self.__keycode_button = gtk.Button("...")
+        self.__keycode_button = Gtk.Button("...")
         self.__keycode_button.connect("clicked", self.__keycode_button_clicked_cb)
         hbox.pack_start(self.__keycode_button, False, True, 4)
         self.pack_start(hbox, False, True, 4)
 
         # modifiers
-        hbox = gtk.HBox()
-        label = gtk.Label(_("Modifiers:"))
-        label.set_justify(gtk.JUSTIFY_LEFT)
+        hbox = Gtk.HBox()
+        label = Gtk.Label(_("Modifiers:"))
+        label.set_justify(Gtk.Justification.LEFT)
         label.set_alignment(0.0, 0.5)
         hbox.pack_start(label, False, True, 4)
 
-        table = gtk.Table(4, 2)
+        table = Gtk.Table(4, 2)
         self.__modifier_buttons = []
-        self.__modifier_buttons.append(("Control",     gtk.CheckButton("_Control"),       gdk.CONTROL_MASK))
-        self.__modifier_buttons.append(("Alt",      gtk.CheckButton("A_lt"),        gdk.MOD1_MASK))
-        self.__modifier_buttons.append(("Shift",    gtk.CheckButton("_Shift"),      gdk.SHIFT_MASK))
-        self.__modifier_buttons.append(("Meta",     gtk.CheckButton("_Meta"),       gdk.META_MASK))
-        self.__modifier_buttons.append(("Super",    gtk.CheckButton("S_uper"),      gdk.SUPER_MASK))
-        self.__modifier_buttons.append(("Hyper",    gtk.CheckButton("_Hyper"),      gdk.HYPER_MASK))
-        self.__modifier_buttons.append(("Capslock", gtk.CheckButton("Capsloc_k"),   gdk.LOCK_MASK))
-        self.__modifier_buttons.append(("Release",  gtk.CheckButton("_Release"),    gdk.RELEASE_MASK))
+        self.__modifier_buttons.append(("Control",
+                                        Gtk.CheckButton.new_with_mnemonic("_Control"),
+                                        Gdk.ModifierType.CONTROL_MASK))
+        self.__modifier_buttons.append(("Alt",
+                                        Gtk.CheckButton.new_with_mnemonic("A_lt"),
+                                        Gdk.ModifierType.MOD1_MASK))
+        self.__modifier_buttons.append(("Shift",
+                                        Gtk.CheckButton.new_with_mnemonic("_Shift"),
+                                        Gdk.ModifierType.SHIFT_MASK))
+        self.__modifier_buttons.append(("Meta",
+                                        Gtk.CheckButton.new_with_mnemonic("_Meta"),
+                                        Gdk.ModifierType.META_MASK))
+        self.__modifier_buttons.append(("Super",
+                                        Gtk.CheckButton.new_with_mnemonic("S_uper"),
+                                        Gdk.ModifierType.SUPER_MASK))
+        self.__modifier_buttons.append(("Hyper",
+                                        Gtk.CheckButton.new_with_mnemonic("_Hyper"),
+                                        Gdk.ModifierType.HYPER_MASK))
+        # <CapsLock> is not parsed by gtk_accelerator_parse()
+        # FIXME: Need to check if ibus gtk panel can enable <Release>.
+        self.__modifier_buttons.append(("Release",
+                                        Gtk.CheckButton.new_with_mnemonic("_Release"),
+                                        Gdk.ModifierType.RELEASE_MASK))
         for name, button, mask in self.__modifier_buttons:
             button.connect("toggled", self.__modifier_button_toggled_cb, name)
 
@@ -100,27 +117,26 @@ class KeyboardShortcutSelection(gtk.VBox):
         table.attach(self.__modifier_buttons[4][1], 0, 1, 1, 2)
         table.attach(self.__modifier_buttons[5][1], 1, 2, 1, 2)
         table.attach(self.__modifier_buttons[6][1], 2, 3, 1, 2)
-        table.attach(self.__modifier_buttons[7][1], 3, 4, 1, 2)
         hbox.pack_start(table, True, True, 4)
         self.pack_start(hbox, False, True, 4)
 
         # buttons
-        hbox = gtk.HBox()
+        hbox = Gtk.HBox()
         # add button
-        self.__add_button = gtk.Button(stock = gtk.STOCK_ADD)
+        self.__add_button = Gtk.Button(stock = Gtk.STOCK_ADD)
         self.__add_button.set_sensitive(False)
         self.__add_button.connect("clicked", self.__add_button_clicked_cb)
-        hbox.pack_start(self.__add_button)
+        hbox.pack_start(self.__add_button, False, True, 0)
         # apply button
-        self.__apply_button = gtk.Button(stock = gtk.STOCK_APPLY)
+        self.__apply_button = Gtk.Button(stock = Gtk.STOCK_APPLY)
         self.__apply_button.set_sensitive(False)
         self.__apply_button.connect("clicked", self.__apply_button_clicked_cb)
-        hbox.pack_start(self.__apply_button)
+        hbox.pack_start(self.__apply_button, False, True, 0)
         # delete button
-        self.__delete_button = gtk.Button(stock = gtk.STOCK_DELETE)
+        self.__delete_button = Gtk.Button(stock = Gtk.STOCK_DELETE)
         self.__delete_button.set_sensitive(False)
         self.__delete_button.connect("clicked", self.__delete_button_clicked_cb)
-        hbox.pack_start(self.__delete_button)
+        hbox.pack_start(self.__delete_button, False, True, 0)
         self.pack_start(hbox, False, True, 4)
 
     def set_shortcuts(self, shortcuts = None):
@@ -132,7 +148,8 @@ class KeyboardShortcutSelection(gtk.VBox):
         added = []
         for shortcut in shortcuts:
             if shortcut not in added:
-                model.insert(-1, (shortcut,))
+                it = model.insert(0)
+                model[it][0] = shortcut
                 added.append(shortcut)
 
     def get_shortcuts(self):
@@ -148,15 +165,16 @@ class KeyboardShortcutSelection(gtk.VBox):
             return
         if shortcut in self.get_shortcuts():
             return
-        iter = model.insert(-1, (shortcut,))
+        it = model.insert(0)
+        model[it][0] = shortcut
         self.__add_button.set_sensitive(False)
-        path = model.get_path(iter)
-        self.__shortcut_view.set_cursor(path)
+        path = model.get_path(it)
+        self.__shortcut_view.set_cursor(path, None, False)
 
     def __get_shortcut_from_buttons(self):
         modifiers = []
         keycode = self.__keycode_entry.get_text()
-        if gdk.keyval_from_name(keycode) == 0:
+        if Gdk.keyval_from_name(keycode) == 0:
             return None
 
         for name, button, mask in self.__modifier_buttons:
@@ -164,26 +182,27 @@ class KeyboardShortcutSelection(gtk.VBox):
                 modifiers.append(name)
         if keycode.startswith("_"):
             keycode = keycode[1:]
-        keys = modifiers + [keycode]
-        shortcut = "+".join(keys)
+        shortcut = "".join(map(lambda m: '<' + m + '>', modifiers))
+        shortcut += keycode
         return shortcut
 
     def __set_shortcut_to_buttons(self, shortcut):
-        keys = shortcut.split("+")
-        mods = keys[:-1]
+        (keyval, state) = Gtk.accelerator_parse(shortcut)
+        if keyval == 0 and state == 0:
+            return
         for name, button, mask in self.__modifier_buttons:
-            if name in mods:
+            if state & mask:
                 button.set_active(True)
             else:
                 button.set_active(False)
-        self.__keycode_entry.set_text(keys[-1])
+        self.__keycode_entry.set_text(shortcut.rsplit('>', 1)[-1])
 
     def __get_selected_shortcut(self):
         model = self.__shortcut_view.get_model()
         path, column = self.__shortcut_view.get_cursor()
         if path == None:
             return None
-        return model[path[0]][0]
+        return model[path.get_indices()[0]][0]
 
     def __set_selected_shortcut(self, shortcut):
         model = self.__shortcut_view.get_model()
@@ -194,13 +213,13 @@ class KeyboardShortcutSelection(gtk.VBox):
     def __del_selected_shortcut(self):
         model = self.__shortcut_view.get_model()
         path, column = self.__shortcut_view.get_cursor()
-        del model[path[0]]
+        model.remove(model.get_iter(path))
         self.__update_add_and_apply_buttons()
 
     def __shortcut_view_cursor_changed_cb(self, treeview):
         shortcut = self.__get_selected_shortcut()
-        self.__set_shortcut_to_buttons(shortcut)
         if shortcut != None:
+            self.__set_shortcut_to_buttons(shortcut)
             self.__delete_button.set_sensitive(True)
         else:
             self.__delete_button.set_sensitive(False)
@@ -227,51 +246,56 @@ class KeyboardShortcutSelection(gtk.VBox):
 
     def __keycode_button_clicked_cb(self, button):
         out = []
-        dlg = gtk.MessageDialog(parent = self.get_toplevel(), buttons = gtk.BUTTONS_CLOSE)
+        dlg = Gtk.MessageDialog(parent = self.get_toplevel(), buttons = Gtk.ButtonsType.CLOSE)
         message = _("Please press a key (or a key combination).\nThe dialog will be closed when the key is released.")
         dlg.set_markup(message)
         dlg.set_title(_("Please press a key (or a key combination)"))
+        sw = Gtk.ScrolledWindow()
 
-        def __key_press_event(d, k, out):
-            out.append(k.copy())
+        def __accel_edited_cb(c, path, keyval, state, keycode):
+            out.append(keyval)
+            out.append(state)
+            out.append(keycode)
+            dlg.response(Gtk.ResponseType.OK)
 
-        def __key_release_event(d, k, out):
-            d.response(gtk.RESPONSE_OK)
-
-        dlg.connect("key-press-event", __key_press_event, out)
-        dlg.connect("key-release-event", __key_release_event, None)
+        model = Gtk.ListStore(GObject.TYPE_INT,
+                              GObject.TYPE_UINT,
+                              GObject.TYPE_UINT)
+        accel_view = Gtk.TreeView(model)
+        sw.add(accel_view)
+        column = Gtk.TreeViewColumn()
+        renderer = Gtk.CellRendererAccel(accel_mode=Gtk.CellRendererAccelMode.OTHER,
+                                         editable=True)
+        renderer.connect('accel-edited', __accel_edited_cb)
+        column.pack_start(renderer, True)
+        column.add_attribute(renderer, 'accel-mods', 0)
+        column.add_attribute(renderer, 'accel-key', 1)
+        column.add_attribute(renderer, 'keycode', 2)
+        accel_view.append_column(column)
+        it = model.append(None)
+        area = dlg.get_message_area()
+        area.pack_end(sw, True, True, 0)
+        sw.show_all()
         id = dlg.run()
         dlg.destroy()
-        if id != gtk.RESPONSE_OK or not out:
+        if id != Gtk.ResponseType.OK or len(out) < 3:
             return
-        keyevent = out[len(out) - 1]
-        state = keyevent.state & (gdk.CONTROL_MASK | \
-                                  gdk.SHIFT_MASK   | \
-                                  gdk.MOD1_MASK    | \
-                                  gdk.META_MASK    | \
-                                  gdk.SUPER_MASK   | \
-                                  gdk.HYPER_MASK)
-        if state == 0:
-            state = state | gdk.RELEASE_MASK
-        elif keyevent.keyval in (keysyms.Control_L, keysyms.Control_R) and state == gdk.CONTROL_MASK:
-            state = state | gdk.RELEASE_MASK
-        elif keyevent.keyval in (keysyms.Shift_L, keysyms.Shift_R) and state == gdk.SHIFT_MASK:
-            state = state | gdk.RELEASE_MASK
-        elif keyevent.keyval in (keysyms.Alt_L, keysyms.Alt_R) and state == gdk.MOD1_MASK:
-            state = state | gdk.RELEASE_MASK
-        elif keyevent.keyval in (keysyms.Meta_L, keysyms.Meta_R) and state == gdk.META_MASK:
-            state = state | gdk.RELEASE_MASK
-        elif keyevent.keyval in (keysyms.Super_L, keysyms.Super_R) and state == gdk.SUPER_MASK:
-            state = state | gdk.RELEASE_MASK
-        elif keyevent.keyval in (keysyms.Hyper_L, keysyms.Hyper_R) and state == gdk.HYPER_MASK:
-            state = state | gdk.RELEASE_MASK
+        keyval = out[0]
+        state = out[1]
+        keycode = out[2]
 
         for name, button, mask in self.__modifier_buttons:
             if state & mask:
                 button.set_active(True)
             else:
                 button.set_active(False)
-        self.__keycode_entry.set_text(gdk.keyval_name(keyevent.keyval))
+
+        shortcut = Gtk.accelerator_name_with_keycode(None,
+                                                     keyval,
+                                                     keycode,
+                                                     state)
+        shortcut = shortcut.replace('<Primary>', '<Control>')
+        self.__keycode_entry.set_text(shortcut.rsplit('>', 1)[-1])
 
     def __add_button_clicked_cb(self, button):
         shortcut = self.__get_shortcut_from_buttons()
@@ -286,11 +310,11 @@ class KeyboardShortcutSelection(gtk.VBox):
         self.__delete_button.set_sensitive(False)
         self.__apply_button.set_sensitive(False)
 
-class KeyboardShortcutSelectionDialog(gtk.Dialog):
+class KeyboardShortcutSelectionDialog(Gtk.Dialog):
     def __init__(self, title = None, parent = None, flags = 0, buttons = None):
         super(KeyboardShortcutSelectionDialog, self).__init__(title, parent, flags, buttons)
         self.__selection_view = KeyboardShortcutSelection()
-        self.vbox.pack_start(self.__selection_view)
+        self.vbox.pack_start(self.__selection_view, False, True, 0)
         self.vbox.show_all()
 
     def set_shortcuts(self, shotrcuts = None):
@@ -307,7 +331,8 @@ class KeyboardShortcutSelectionDialog(gtk.Dialog):
 if __name__ == "__main__":
     dlg = KeyboardShortcutSelectionDialog(
         title = "Select test",
-        buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK))
+        buttons = (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
+                   Gtk.STOCK_OK, Gtk.ResponseType.OK))
     dlg.add_shortcut("Control+Shift+space")
     dlg.set_shortcuts(None)
     print dlg.run()
